package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.network.chat.Component
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.MenuProvider
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.phys.AABB
import net.minecraftforge.common.ForgeConfigSpec
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.core.Decimal
import ru.dbotthepony.mc.otm.core.DecimalConfigValue
import ru.dbotthepony.mc.otm.core.defineDecimal
import ru.dbotthepony.mc.otm.core.ifPresentK
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MNames
import ru.dbotthepony.mc.otm.core.WriteOnce

@Suppress("ObjectPropertyName")
class AndroidStationBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) :
	MatteryPoweredBlockEntity(MBlockEntities.ANDROID_STATION, p_155229_, p_155230_), MenuProvider {

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return AndroidStationMenu(containerID, inventory, this)
	}

	override val defaultDisplayName: Component
		get() = MBlocks.ANDROID_STATION.name

	override val energy = object : WorkerEnergyStorage(this@AndroidStationBlockEntity::setChangedLight, ::CAPACITY, ::MAX_IO, { null }) {
		override fun extractEnergyInner(howMuch: Decimal, simulate: Boolean): Decimal {
			return super.extractEnergyInner(howMuch, simulate).also {
				if (!simulate && this.batteryLevel.isZero) {
					if (blockState.getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.IDLE) {
						level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE), Block.UPDATE_CLIENTS)
					}
				}
			}
		}

		override fun receiveEnergyInner(howMuch: Decimal, simulate: Boolean): Decimal {
			return super.receiveEnergyInner(howMuch, simulate).also {
				if (!simulate && it.isPositive) {
					if (blockState.getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.WORKING) {
						level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.WORKING), Block.UPDATE_CLIENTS)
					}
				}
			}
		}
	}

	private var tickedOnve = false

	fun tick() {
		if (!tickedOnve) {
			tickedOnve = true

			if (energy.batteryLevel.isPositive && blockState.getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.WORKING) {
				level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.WORKING), Block.UPDATE_CLIENTS)
			} else if (energy.batteryLevel.isZero && blockState.getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.IDLE) {
				level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE), Block.UPDATE_CLIENTS)
			}
		}

		batteryChargeLoop()

		if (isBlockedByRedstone) return
		val level = level ?: return
		val x = blockPos.x.toDouble()
		val y = blockPos.y.toDouble()
		val z = blockPos.z.toDouble()

		for (ent in level.getEntitiesOfClass(ServerPlayer::class.java, AABB(x, y, z, x + 1.0, y + 2.0, z + 1.0))) {
			ent.getCapability(MatteryCapability.MATTERY_PLAYER).ifPresentK {
				if (!it.isAndroid)
					return@ifPresentK

				val missing = it.androidEnergy.missingPower

				if (missing > Decimal.ZERO) {
					val extract = energy.extractEnergyInner(missing, true)

					if (extract > Decimal.ZERO) {
						val received = it.androidEnergy.receiveEnergyOuter(extract, false)
						energy.extractEnergyInner(received, false)
					}
				}
			}
		}
	}

	companion object {
		private var _CAPACITY: DecimalConfigValue by WriteOnce()
		private var _MAX_IO: DecimalConfigValue by WriteOnce()
		private var _ENERGY_PER_OPERATION: DecimalConfigValue by WriteOnce()
		private var _ENERGY_PER_RESEARCH: DecimalConfigValue by WriteOnce()

		val CAPACITY get() = _CAPACITY.get()
		val MAX_IO get() = _MAX_IO.get()
		val ENERGY_PER_OPERATION get() = _ENERGY_PER_OPERATION.get()
		val ENERGY_PER_RESEARCH get() = _ENERGY_PER_RESEARCH.get()

		fun registerConfig(builder: ForgeConfigSpec.Builder) {
			builder.push(MNames.ANDROID_STATION)

			_CAPACITY = builder.defineDecimal("capacity", Decimal(100_000), Decimal.ONE)
			_MAX_IO = builder.defineDecimal("throughput", Decimal.valueOf(2048), Decimal.ONE)
			_ENERGY_PER_OPERATION = builder.comment("Swapping parts, etc").defineDecimal("energyPerOperation", Decimal.valueOf(2048), Decimal.ONE)
			_ENERGY_PER_RESEARCH = builder.defineDecimal("energyPerResearch", Decimal.valueOf(16384), Decimal.ONE)

			builder.pop()
		}
	}
}
