package ru.dbotthepony.mc.otm.client.screen.panels

import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.gui.screens.Screen
import net.minecraft.client.gui.screens.inventory.AbstractContainerScreen
import net.minecraft.client.gui.screens.inventory.InventoryScreen
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.entity.player.Player
import ru.dbotthepony.mc.otm.capability.matteryPlayer
import ru.dbotthepony.mc.otm.client.render.Widgets8
import ru.dbotthepony.mc.otm.client.render.element
import ru.dbotthepony.mc.otm.client.screen.ExoPackInventoryScreen
import ru.dbotthepony.mc.otm.client.screen.panels.buttons.SmallBooleanRectangleButtonPanel
import ru.dbotthepony.mc.otm.compat.cos.CosmeticToggleRenderButton
import ru.dbotthepony.mc.otm.compat.cos.isCosmeticArmorLoaded
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.asGetterOnly
import ru.dbotthepony.mc.otm.network.DisplayExosuitPacket
import ru.dbotthepony.mc.otm.network.HideExosuitPacket
import ru.dbotthepony.mc.otm.network.MatteryPlayerNetworkChannel

private fun calculateScale(width: Float, height: Float): Int {
	val aspectRatio = width / height

	if (aspectRatio.isNaN()) {
		return 1
	}

	if (aspectRatio < 0.7083) {
		return (width / 51f * 30f).toInt()
	} else {
		return (height / 72f * 30f).toInt()
	}
}

class EntityRendererPanel<out S : Screen> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>?,
	val entity: LivingEntity,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = ENTITY_RECTANGLE.width,
	height: Float = ENTITY_RECTANGLE.height,
	var renderScale: Int = calculateScale(width, height)
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	init {
		dockPadding = DockProperty(1f, 1f, 1.5f, 1f)
	}

	val canvas = object : EditablePanel<S>(screen, this@EntityRendererPanel) {
		init {
			dock = Dock.FILL
			scissor = true
		}

		private val cosButton: EditablePanel<S>?
		private val exosuitButton: EditablePanel<S>?

		init {
			if (entity is Player && isCosmeticArmorLoaded) {
				cosButton = CosmeticToggleRenderButton(screen, this, x = this.width - 7f, y = this.height - 7f)
			} else {
				cosButton = null
			}

			if (entity is Player) {
				val matteryPlayer = entity.matteryPlayer

				if (matteryPlayer != null && matteryPlayer.hasExoPack) {
					exosuitButton = SmallBooleanRectangleButtonPanel(screen, this, this.width - 2f - SmallBooleanRectangleButtonPanel.SIZE, 2f,
						prop = matteryPlayer::displayExoPack.asGetterOnly(),
						skinElementActive = Widgets8.EXOSUIT_SHOWN,
						skinElementInactive = Widgets8.EXOSUIT_HIDDEN,
						onChange = {
							if (it) {
								MatteryPlayerNetworkChannel.sendToServer(DisplayExosuitPacket)
							} else {
								MatteryPlayerNetworkChannel.sendToServer(HideExosuitPacket)
							}
						}
					)

					exosuitButton.tooltip = TranslatableComponent("otm.gui.exosuit.toggle_visibility")
				} else {
					exosuitButton = null
				}
			} else {
				exosuitButton = null
			}
		}

		override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
			if (entity.isDeadOrDying) {
				return
			}

			val renderX = absoluteX.toInt() + width.toInt() / 2
			val renderY = absoluteY.toInt() + (height * 0.9f).toInt()

			InventoryScreen.renderEntityInInventory(
				renderX,
				renderY,
				renderScale,
				renderX - mouseX,
				absoluteY + height * 0.15f - mouseY,
				entity
			)
		}

		override fun performLayout() {
			super.performLayout()

			if (cosButton != null) {
				cosButton.x = this.width - 7f
				cosButton.y = this.height - 7f
			}

			if (exosuitButton != null) {
				exosuitButton.x = this.width - 2f - exosuitButton.width
				exosuitButton.y = 2f
			}
		}
	}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		ExoPackInventoryScreen.ENTITY_RECTANGLE.render(stack)
	}

	companion object {
		val ENTITY_RECTANGLE = AbstractContainerScreen.INVENTORY_LOCATION.element(25f, 7f, 51f, 72f)
	}
}
